@@ -141,7 +141,7 @@ fitted.smooth <- function(object, ...){
 }
 
 forecast.smooth <- function(object, h=10,
-                            intervals=c("none","parametric","semiparametric","nonparametric"),
+                            intervals=c("parametric","semiparametric","nonparametric","none"),
                             level=0.95, ...){
     intervals <- intervals[1];
     if(gregexpr("ETS",object$model)!=-1){
@@ -207,6 +207,18 @@ modelType.default <- function(object, ...){
         }
         else if(gregexpr("CES",model)!=-1){
             modelType <- substring(model,unlist(gregexpr("\\(",model))+1,unlist(gregexpr("\\)",model))-1);
+            if(modelType=="n"){
+                modelType <- "none";
+            }
+            else if(modelType=="s"){
+                modelType <- "simple";
+            }
+            else if(modelType=="p"){
+                modelType <- "partial";
+            }
+            else{
+                modelType <- "full";
+            }
         }
         else{
             modelType <- NA;
@@ -299,7 +311,7 @@ plot.smooth.sim <- function(x, ...){
     }
 
     if(nsim==1){
-        plot(x$data, main=x$model);
+        plot(x$data, main=x$model, ylab="Data");
     }
     else{
         message(paste0("You have generated ",nsim," time series. Not sure which of them to plot.\n",
@@ -442,6 +454,18 @@ print.smooth.sim <- function(x, ...){
             }
             cat(paste0("True likelihood: ",round(x$likelihood,3),"\n"));
         }
+        else if(gregexpr("CES",x$model)!=-1){
+            cat(paste0("Smoothing parameter A: ",round(x$A,3),"\n"));
+            if(!is.null(x$B)){
+                if(is.complex(x$B)){
+                    cat(paste0("Smoothing parameter B: ",round(x$B,3),"\n"));
+                }
+                else{
+                    cat(paste0("Smoothing parameter b: ",round(x$B,3),"\n"));
+                }
+            }
+            cat(paste0("True likelihood: ",round(x$likelihood,3),"\n"));
+        }
     }
 }
 
@@ -510,7 +534,6 @@ simulate.smooth <- function(object, nsim=1, seed=NULL, obs=NULL, ...){
         }
     }
     else if(gregexpr("ARIMA",object$model)!=-1){
-        model <- object$model;
         orders <- orders(object);
         lags <- lags(object);
         randomizer <- "rnorm";
@@ -519,6 +542,15 @@ simulate.smooth <- function(object, nsim=1, seed=NULL, obs=NULL, ...){
                                      initial=object$initial, obs=obs, nsim=nsim, silent=TRUE,
                                      iprob=object$iprob[length(object$iprob)], randomizer=randomizer, mean=0, sd=sqrt(object$s2),...);
     }
+    else if(gregexpr("CES",object$model)!=-1){
+        model <- substring(object$model,unlist(gregexpr("\\(",object$model))+1,unlist(gregexpr("\\)",object$model))-1);
+        initial <- object$initial;
+        randomizer <- "rnorm";
+        simulatedData <- sim.ces(seasonality=model,
+                                 frequency=frequency(object$actuals), A=object$A, B=object$B,
+                                 initial=object$initial, obs=obs, nsim=nsim, silent=TRUE,
+                                 iprob=object$iprob[length(object$iprob)], randomizer=randomizer, mean=0, sd=sqrt(object$s2),...);
+    }
     else{
         model <- substring(object$model,1,unlist(gregexpr("\\(",object$model))[1]-1);
         message(paste0("Sorry, but simulate is not yet available for the model ",model,"."));
